--[[
    SweptRP Chat Edits
    by SweptThrone
    Contact: https://sweptthr.one/contact

    Made for DarkRP.  Unknown support in anything else.

    This module changes most (if not all) of DarkRP's chat interactions to be player-colored instead of job-colored.
    This module also makes changes to various other global chats.
    
    List of edits:
        /advert sends a DarkRP notification about /advert being disabled.
        /all re-implements /advert's old yellow text functionality.
        /ad re-implements /advert's old yellow text functionality.
        /ic re-implements /advert's old yellow text functionality.
            These are done so that raid/mug/&c binds don't work while still allowing a method of in-character global chat.
            Adverts are now prefixed with [IC].
            The color of the player's name is now the player's color instead of their job color.
        The message part of /ooc, /a, and // is now gray instead of white.
            The color of the player's name is now the player's color instead of their job color.
        The color of the player's name in local "regular" chat is now the player's color instead of their job color.
        Team chat (hitting U) now uses DarkRP's radio system (did you even know this existed?)
            Typing /channel # will tune you in to radio channel #.
            Typing in team chat will send an "anonymous" message to everyone tuned in to channel #.
                The sender of these messages shows "Radio #".
                The color of the sender is the player's color with a small amount of random variation.
        DarkRP's /radio command follows the behavior above.
        The color of the player's name in /me chat is now the player's color instead of their job color.
        The color of the player's name in /cr, /911, /999, /112, and /000 messages is now the player's color instead of their job color.
]]--

hook.Add( "InitPostEntity", "SweptRPChatStuff", function()
    DarkRP.removeChatCommand("advert")
    DarkRP.declareChatCommand({
        command = "advert",
        description = "Deprecated.",
        delay = 1.5
    })
    DarkRP.declareChatCommand({
        command = "all",
        description = "Displays an advertisement to everyone in chat.",
        delay = 1.5
    })
    DarkRP.declareChatCommand({
        command = "ad",
        description = "Displays an advertisement to everyone in chat.",
        delay = 1.5
    })
    DarkRP.declareChatCommand({
        command = "ic",
        description = "Displays an advertisement to everyone in chat.",
        delay = 1.5
    })

    if SERVER then
        function newAdvert(ply,args)
            if args == "" then
                DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", "argument", ""))
                return ""
            end
            local DoSay = function(text)
                if text == "" then
                    DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", "argument", ""))
                    return
                end
                for k,v in pairs(player.GetAll()) do
                    local col = ply:GetPlayerColor():ToColor()
                    DarkRP.talkToPerson(v, col, "[IC] " .. ply:Nick(), Color(255, 255, 255, 255), text, ply)
                end
            end
            hook.Call("playerAdverted", nil, ply, args)
            return args, DoSay
        end

        DarkRP.defineChatCommand("all", newAdvert, 1.5)
        DarkRP.defineChatCommand("ad", newAdvert, 1.5 )
        DarkRP.defineChatCommand("ic", newAdvert, 1.5 )
        DarkRP.defineChatCommand("advert",function(ply,args)
            DarkRP.notify(ply, 1, 8, "Advert is disabled on this server.  Use /all, /ad, or /ic instead.  DO NOT USE THIS TO ADVERT CRIMES!")
            return ""
        end, 1.5)
    else
        DarkRP.addChatReceiver("/all", "advertise", function(ply) return true end)
        DarkRP.addChatReceiver("/ad", "advertise", function(ply) return true end)
        DarkRP.addChatReceiver("/ic", "advertise", function(ply) return true end)
    end

    -- ooc
    DarkRP.removeChatCommand("/")
    DarkRP.removeChatCommand("a")
    DarkRP.removeChatCommand("ooc")

    DarkRP.declareChatCommand{
        command = "/",
        description = "Global server chat.",
        delay = 1.5
    }

    DarkRP.declareChatCommand{
        command = "a",
        description = "Global server chat.",
        delay = 1.5
    }

    DarkRP.declareChatCommand{
        command = "ooc",
        description = "Global server chat.",
        delay = 1.5
    }

    if SERVER then

        local function OOC(ply, args)
            if not GAMEMODE.Config.ooc then
                DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("disabled", DarkRP.getPhrase("ooc"), ""))
                return ""
            end

            local DoSay = function(text)
                if text == "" then
                    DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", DarkRP.getPhrase("arguments"), ""))
                    return ""
                end
                local col = ply:GetPlayerColor():ToColor()
                local col2 = Color( 168, 168, 168, 255 )
                if not ply:Alive() then
                    col2 = Color(168, 132, 132, 255)
                    col = col2
                end

                local phrase = DarkRP.getPhrase("ooc")
                local name = ply:Nick()
                for _, v in ipairs(player.GetAll()) do
                    DarkRP.talkToPerson(v, col, "(" .. phrase .. ") " .. name, col2, text, ply)
                end
            end
            return args, DoSay
        end
        DarkRP.defineChatCommand("/", OOC, true, 1.5)
        DarkRP.defineChatCommand("a", OOC, true, 1.5)
        DarkRP.defineChatCommand("ooc", OOC, true, 1.5)
    end

    -- local chat ply col
    function DarkRP.talkToRange(ply, PlayerName, Message, size)
        local ents = player.GetHumans()
        local col = ply:GetPlayerColor():ToColor()
        local filter = {}

        local plyPos = ply:EyePos()
        local sizeSqr = size * size

        for _, v in ipairs(ents) do
            if (v:EyePos():DistToSqr(plyPos) <= sizeSqr) and (v == ply or hook.Run("PlayerCanSeePlayersChat", PlayerName .. ": " .. Message, false, v, ply) ~= false) then
                table.insert(filter, v)
            end
        end

        if PlayerName == ply:Nick() then PlayerName = "" end -- If it's just normal chat, why not cut down on networking and get the name on the client

        net.Start("DarkRP_Chat")
            net.WriteUInt(col.r, 8)
            net.WriteUInt(col.g, 8)
            net.WriteUInt(col.b, 8)
            net.WriteString(PlayerName)
            net.WriteEntity(ply)
            net.WriteUInt(255, 8)
            net.WriteUInt(255, 8)
            net.WriteUInt(255, 8)
            net.WriteString(Message)
        net.Send(filter)
    end

    -- gc
    DarkRP.removeChatCommand( "radio" )
    DarkRP.removeChatCommand( "g" )

    DarkRP.declareChatCommand{
        command = "g",
        description = "Say something through the radio.",
        delay = 0
    }

    DarkRP.declareChatCommand{
        command = "radio",
        description = "Say something through the radio.",
        delay = 0
    }

    if SERVER then

        local function SayThroughRadio(ply,args)
            if not ply.RadioChannel then ply.RadioChannel = 1 end
            local radioChannel = ply.RadioChannel
            if not args or args == "" then
                DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", DarkRP.getPhrase("arguments"), ""))
                return ""
            end
            local DoSay = function(text)
                if text == "" then
                    DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", DarkRP.getPhrase("arguments"), ""))
                    return
                end
                local col = Color(180, 180, 180, 255)
                local pcol = ply:GetPlayerColor():ToColor()
                local rcol = Color( pcol.r + math.random( -32, 32 ), pcol.g + math.random( -32, 32 ), pcol.b + math.random( -32, 32 ), 255 )
                local phrase = DarkRP.getPhrase("radio_x", radioChannel)
                for _, v in ipairs(player.GetAll()) do
                    if v.RadioChannel == radioChannel then
                        DarkRP.talkToPerson(v, rcol, phrase, col, text, ply)
                    end
                end
            end
            return args, DoSay
        end
        
        DarkRP.defineChatCommand("g", SayThroughRadio, 0)
        DarkRP.defineChatCommand("radio", SayThroughRadio, 0)

    end

    -- pm
    DarkRP.removeChatCommand( "pm" )

    DarkRP.declareChatCommand{
        command = "pm",
        description = "Send a private message to someone.",
        delay = 0
    }

    if SERVER then
        local function PM(ply, args)
            local namepos = string.find(args, " ")
            if not namepos then
                DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", DarkRP.getPhrase("arguments"), ""))
                return ""
            end

            local name = string.sub(args, 1, namepos - 1)
            local msg = string.sub(args, namepos + 1)

            if msg == "" then
                DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", DarkRP.getPhrase("arguments"), ""))
                return ""
            end

            local target = DarkRP.findPlayer(name)
            if target == ply then return "" end

            if target then
                local col = ply:GetPlayerColor():ToColor()
                local pname = ply:Nick()
                local col2 = color_white
                DarkRP.talkToPerson(target, col, "(PM) " .. pname, col2, msg, ply)
                DarkRP.talkToPerson(ply, col, "(PM) " .. pname, col2, msg, ply)
            else
                DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("could_not_find", tostring(name)))
            end

            return ""
        end
        DarkRP.defineChatCommand("pm", PM, 1.5)
    else
        DarkRP.addChatReceiver("/pm", "PM", function(ply ,text) 
            if not isstring(text[2]) then return false end
            text[2] = string.lower(tostring(text[2]))

            return string.find(string.lower(ply:Nick()), text[2], 1, true) ~= nil or
                string.find(string.lower(ply:SteamName()), text[2], 1, true) ~= nil or
                string.lower(ply:SteamID()) == text[2]
        end)
    end

    -- me
    DarkRP.removeChatCommand( "me" )

    DarkRP.declareChatCommand{
        command = "me",
        description = "Chat roleplay to say you're doing things that you can't show otherwise.",
        delay = 0
    }

    if SERVER then
        local function Me(ply, args)
            if args == "" then
                DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", DarkRP.getPhrase("arguments"), ""))
                return ""
            end
        
            local DoSay = function(text)
                if text == "" then
                    DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", DarkRP.getPhrase("arguments"), ""))
                    return ""
                end
                if GAMEMODE.Config.alltalk then
                    local col = ply:GetPlayerColor():ToColor()
                    local name = ply:Nick()
                    for _, target in ipairs(player.GetAll()) do
                        DarkRP.talkToPerson(target, col, name .. " " .. text)
                    end
                else
                    DarkRP.talkToRange(ply, ply:Nick() .. " " .. text, "", GAMEMODE.Config.meDistance)
                end
            end
            return args, DoSay
        end
        DarkRP.defineChatCommand("me", Me, 1.5)
    else
        DarkRP.addChatReceiver("/me", DarkRP.getPhrase("perform_your_action"), function(ply) return LocalPlayer():GetPos():DistToSqr(ply:GetPos()) < GAMEMODE.Config.meDistance * GAMEMODE.Config.meDistance end)
    end

    -- 911
    for _, cmd in pairs{"cr", "911", "999", "112", "000"} do
        DarkRP.removeChatCommand( cmd )

        DarkRP.declareChatCommand{
            command = cmd,
            description = "Cry for help, the police will come (hopefully)!",
            delay = 1.5
        }
    end

    if SERVER then
        local function CombineRequest(ply, args)
            if args == "" then
                DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", DarkRP.getPhrase("arguments"), ""))
                return ""
            end
        
            local DoSay = function(text)
                if text == "" then
                    DarkRP.notify(ply, 1, 4, DarkRP.getPhrase("invalid_x", DarkRP.getPhrase("arguments"), ""))
                    return
                end
        
                local col = ply:GetPlayerColor():ToColor()
                local col2 = Color(255, 0, 0, 255)
                local phrase = DarkRP.getPhrase("request")
                local name = ply:Nick()
                for _, v in ipairs(player.GetAll()) do
                    if v:isCP() or v == ply then
                        DarkRP.talkToPerson(v, col, phrase .. " " .. name, col2, text, ply)
                    end
                end
            end
            return args, DoSay
        end
        for _, cmd in ipairs{"cr", "911", "999", "112", "000"} do
            DarkRP.defineChatCommand(cmd, CombineRequest, 1.5)
        end
    end
end )